"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _glMatrix = require("gl-matrix");

var _gridIndex = _interopRequireDefault(require("./grid-index"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var viewportPadding = 100;

var CollisionIndex = function () {
  function CollisionIndex(width, height) {
    _classCallCheck(this, CollisionIndex);

    this.width = void 0;
    this.height = void 0;
    this.grid = void 0;
    this.screenRightBoundary = void 0;
    this.screenBottomBoundary = void 0;
    this.gridRightBoundary = void 0;
    this.gridBottomBoundary = void 0;
    this.width = width;
    this.height = height;
    this.grid = new _gridIndex.default(width + 2 * viewportPadding, height + 2 * viewportPadding, 25);
    this.screenRightBoundary = width + viewportPadding;
    this.screenBottomBoundary = height + viewportPadding;
    this.gridRightBoundary = width + 2 * viewportPadding;
    this.gridBottomBoundary = height + 2 * viewportPadding;
  }

  _createClass(CollisionIndex, [{
    key: "placeCollisionBox",
    value: function placeCollisionBox(collisionBox, mvpMatrix) {
      var projectedPoint = this.project(mvpMatrix, collisionBox.anchorPointX, collisionBox.anchorPointY);
      var tlX = collisionBox.x1 + projectedPoint.x;
      var tlY = collisionBox.y1 + projectedPoint.y;
      var brX = collisionBox.x2 + projectedPoint.x;
      var brY = collisionBox.y2 + projectedPoint.y;

      if (!this.isInsideGrid(tlX, tlY, brX, brY) || this.grid.hitTest(tlX, tlY, brX, brY)) {
        return {
          box: []
        };
      }

      return {
        box: [tlX, tlY, brX, brY]
      };
    }
  }, {
    key: "insertCollisionBox",
    value: function insertCollisionBox(box, featureIndex) {
      var key = {
        featureIndex: featureIndex
      };
      this.grid.insert(key, box[0], box[1], box[2], box[3]);
    }
  }, {
    key: "project",
    value: function project(mvpMatrix, x, y) {
      var point = _glMatrix.vec4.fromValues(x, y, 0, 1);

      var out = _glMatrix.vec4.create();

      _glMatrix.vec4.transformMat4(out, point, mvpMatrix);

      return {
        x: (out[0] / out[3] + 1) / 2 * this.width + viewportPadding,
        y: (-out[1] / out[3] + 1) / 2 * this.height + viewportPadding
      };
    }
  }, {
    key: "isInsideGrid",
    value: function isInsideGrid(x1, y1, x2, y2) {
      return x2 >= 0 && x1 < this.gridRightBoundary && y2 >= 0 && y1 < this.gridBottomBoundary;
    }
  }]);

  return CollisionIndex;
}();

exports.default = CollisionIndex;
//# sourceMappingURL=collision-index.js.map